import warnings
warnings.filterwarnings('ignore')
import pylab
import os
import pandas as pd
import obspy
import math
import numpy as np
import matplotlib.pyplot as plt
from obspy import read
pylab.rcParams['figure.figsize'] = (12.0, 8.0)
from obspy import UTCDateTime
from obspy.clients.filesystem.sds import Client
root = '/etrepo/data-sites/sosenattos/ETSCOPE'
archive = Client(root)
from obspy.io.xseed import Parser
from obspy.signal import PPSD
from obspy import read_inventory
from numpy import linalg as eigen
import glob
from datetime import datetime
from scipy.signal import savgol_filter
import cmath
from obspy.signal import konnoohmachismoothing

inv  = read_inventory('*mettere path a inventory file per borehole*')

Start = UTCDateTime('201-9-1T00:00')      

windowLength = 3600
windowShift      = int(windowLength * 0.5)

real_FFT1 = []
real_FFT2 = []
real_FFT3 = []

# Data di stop può essere messa a piacere

while Start <= UTCDateTime('2021-9-2T23:59'):
    
    print(Start)
    
    st = client.get_waveforms(station='P2',channel='HH?',network='ET',location='00',starttime=Start,endtime=Start + 3600)
    st.remove_response(output="VEL", water_level=60,inventory=inv)
    
    tre = st[0]
    trn = st[1]
    trz = st[2]
    
    delta = float(tre.stats.delta)
    samp = trz.stats.sampling_rate
    
    nSegments      = 15  # total number of segments to calculate FFT for a window                                                                       
    percentOverlap = 50  # percent segment overlap                                                                                                      
    nSegWindow     = int((nSegments-1) * (1.0 - float(percentOverlap)/100.0)) + 1 
    thisNSamp      = int((windowLength/delta + 1)/nSegWindow)
    nSamp          = 2**int(math.log(thisNSamp, 2))
    windowShift    = int(windowLength * 0.5)
    taperWindow    = np.hanning(nSamp)
    nShift = int(nSamp * (1.0 - float(percentOverlap) / 100.0))
    startSamp = 0
    endSamp = 0
    nSegCount = 0
    frequency = np.array(np.arange(0,int(nSamp/2)+1)/float(nSamp * delta))
    
    m11 = np.zeros(int(nSamp/2)+1,dtype=np.complex)
    m22 = np.zeros(int(nSamp/2)+1,dtype=np.complex)
    m33 = np.zeros(int(nSamp/2)+1,dtype=np.complex)

    FFT1 = np.zeros(int(nSamp/2)+1, dtype=np.complex)
    FFT2 = np.zeros(int(nSamp/2)+1, dtype=np.complex)
    FFT3 = np.zeros(int(nSamp/2)+1, dtype=np.complex)
    
    for i in range(nSegments):
                   
        try:
            
            endSamp = startSamp + nSamp
    
            tapered_dataz = trz.data[startSamp:endSamp]
            tapered_datae = tre.data[startSamp:endSamp]
            tapered_datan = trn.data[startSamp:endSamp]

            nSegCount += 1

            tapered_dataz = tapered_dataz - np.mean(tapered_dataz)
            tapered_datae = tapered_datae - np.mean(tapered_datae)
            tapered_datan = tapered_datan - np.mean(tapered_datan)

            tapered_dataz = tapered_dataz * taperWindow
            tapered_datae = tapered_datae * taperWindow
            tapered_datan = tapered_datan * taperWindow

            FFT1 = np.fft.rfft(tapered_datae)
            FFT2 = np.fft.rfft(tapered_datan)
            FFT3 = np.fft.rfft(tapered_dataz)

            m11 += FFT1[0:int(nSamp/2)+1]
            m22 += FFT2[0:int(nSamp/2)+1]
            m33 += FFT3[0:int(nSamp/2)+1]

            startSamp += nShift
            
        except:
            
            print('Tapering fail - skipping segment')
        
    real_FFT1.append(abs(m11))
    real_FFT2.append(abs(m22))
    real_FFT3.append(abs(m33))
    
    Start += 3600
    Stop += 3600
    
print(np.shape(real_FFT1))
smoothed1 = konnoohmachismoothing.konno_ohmachi_smoothing(np.array(real_FFT1, dtype=np.float32), frequency, bandwidth=20, count=2, normalize=True)
smoothed2 = konnoohmachismoothing.konno_ohmachi_smoothing(np.array(real_FFT2, dtype=np.float32), frequency, bandwidth=20, count=2, normalize=True)
smoothed3 = konnoohmachismoothing.konno_ohmachi_smoothing(np.array(real_FFT3, dtype=np.float32), frequency, bandwidth=20, count=2, normalize=True)

print(len(real_FFT1))
print(np.shape(smoothed1))
print(np.shape(real_FFT3))
          
H_comb = (np.sqrt(smoothed2 * smoothed1))

HV_ratio = (H_comb / smoothed3)

print(np.shape(H_comb))    
print(np.shape(HV_ratio))
    
    
pylab.rcParams['figure.figsize'] = (10.0, 6.0)
pylab.rcParams.update({'font.size': 25})
plt.plot(frequency, np.mean(H_comb, axis = 0), label = 'H+V', linewidth = 2.5)
plt.plot(frequency, np.mean(smoothed3, axis = 0), label = 'Z', linewidth = 2.5)
plt.plot(frequency, np.mean(smoothed1, axis = 0), label = 'E', linewidth = 2.5)
plt.plot(frequency, np.mean(smoothed2, axis = 0), label = 'N', linewidth = 2.5)
plt.xlim([0.01,10])
plt.xscale('log')
plt.yscale('log')
plt.xlabel('Frequency [Hz]')
plt.ylabel('ASD [m/s^2/Hz^1/2]')
plt.title('Average ASD (FFT) - SOE0')
plt.grid(True,which="both",ls="-")
plt.legend()
plt.savefig('/homes/digiovanni/Plots/average_ASD_FFT_SOE2.png', bbox_inches = "tight")
plt.show()


percentile_95 = np.nanpercentile(HV_ratio, [95], axis = 0)
percentile_5 = np.nanpercentile(HV_ratio, [5], axis = 0)
percentile_50 = np.nanpercentile(HV_ratio, [50], axis = 0)

plt.clf()
plt.plot(frequency, np.nanmean(HV_ratio, axis = 0), color = 'k', label = 'HVSR average', linewidth = 2.5)
plt.plot(frequency, percentile_95[0], label = 'Percentiles', color = 'k', linestyle = 'dashed', linewidth = 2.5)
plt.plot(frequency, percentile_5[0], color = 'k', linestyle = 'dashed', linewidth = 2.5)
plt.xlim([0.1,10])
plt.ylim([0.4,10])
plt.xlabel('Frequency [Hz]')
plt.ylabel('HVSR ratio')
#plt.title('HVSR ratio')
plt.grid(True, which="both", axis = 'x',ls="-")
plt.grid(True, which="major", axis = 'y', ls="-")
plt.xscale('log')
plt.yscale('log')
plt.legend()
plt.savefig('/homes/digiovanni/Plots/HV_ratio_SOE2_straight_FFT_1_week.png', bbox_inches = "tight")
plt.show()
