import sys
import argparse
import warnings
import numpy as np
from scipy.signal import spectrogram
from obspy import UTCDateTime
from obspy import read_inventory
from obspy.clients.fdsn.header import URL_MAPPINGS
from multiprocessing.pool import ThreadPool
from functools import partial
warnings.filterwarnings('ignore')
sys.path.append('/homes/digiovanni/my_python_modules')
from seismo_util import plot_n_save
from obspy.clients.filesystem.sds import Client
from obspy.clients.fdsn import Client as remoto

def input_spectra(cha, args, inv, Start, i):
    
    Start = UTCDateTime(Start)
    if args.source == 'local':
                
        client = Client(args.client)
        
    elif args.source == 'fdsn':
        
        client  = remoto(args.client)
        
    st = client.get_waveforms(station=args.sta,channel=cha,network=args.net,location=args.loc,starttime=Start,endtime=Start + 86400)
    #fs = int(st[0].stats.sampling_rate)
    
    if len(st) == 1:

        fs = int(st[0].stats.sampling_rate)
        st.remove_response(output="VEL", water_level=60,inventory=inv)
        
        ff, tt, S = spectrogram(np.array(st[0].data), fs=fs, window='hann', nperseg=fs*args.T_fft, noverlap=int(fs*args.T_fft//2))
        plot     = False
        
        if i == 0: 
                        
            a, b      = np.shape(S)
            S_tot     = np.empty((a, 0), dtype = 'float64')
            S_tot     = np.append(S_tot, S, axis=1)
                                              
        elif i >= 1:
                        
            try:
                S_tot
    
            except NameError:
                
                a,b        = np.shape(S)
                S_tot     = np.empty((a, 0), dtype = 'float64')  
                             
            S_tot     = np.append(S_tot, S, axis=1)
            
        if i == args.days - 1: plot = False
        
        bb      = np.logspace(-11, -5, 200)
        plot_n_save(args.datetime, args.days, bb, ff, tt, args.T_fft, S_tot, args.sta, args.loc, cha, write = False, plot = plot)
    
    else:
        
        print('SKIP SEGMENT')

if __name__=="__main__":
    
    
    parser = argparse.ArgumentParser()
    parser.add_argument('-source',help='source of the data (local/fdsn)',action='store',type=str)
    parser.add_argument('-client',help='path to SDS directory (local) or fdsn client',action='store',type=str)
    parser.add_argument('-net',help='network code',action='store',type=str)
    parser.add_argument('-sta',help='network code',action='store',type=str)
    parser.add_argument('-loc',help='network code',action='store',type=str)
    parser.add_argument('-cha',help='channel code',action='store',type=str)
    parser.add_argument('-datetime',help='starttime of the analysis',action='store',type=str)
    parser.add_argument('-days',help='number of days to analyze',action='store',type=int)
    parser.add_argument('-T_fft',help='length of FFT in seconds',action='store',type=int)
    args = parser.parse_args()
    
    if args.loc == None:
    
        args.loc = ''
    
    if args.source == 'local':

        inv    = read_inventory('/homes/digiovanni/dataless/cjunkk_ETScope-2.xml')
        inv   += read_inventory('/homes/digiovanni/dataless/Windpark_Array_2023.xml')
        inv   += read_inventory('/archive/SARARRAY/dataless/7P_network.xml')
        inv   += read_inventory('/homes/digiovanni/dataless/KB_DZA_fdsn.xml')
        
    if args.source == 'fdsn':
        
        client  = remoto(args.client)
        inv     = client.get_stations(network=args.net, station=args.sta, loc=args.loc, channel='HH*', level="response")
            
    Start_0         = UTCDateTime(args.datetime) 
    
    if args.T_fft == None: 
        
        args.T_fft = 128
        
    if args.cha == None: 
        
        cha = 'HH*'
        
    else:
        
        cha = args.cha
        
    bb              = np.logspace(-11, -5, 200)
    S_tot           = 0
    plot            = False
    
    for i in range(args.days):
           
        Start = Start_0 + i*86400
        print('Processing day ', Start)
         
        
        if cha == 'HH*' :
            
            n_processor = 3
            with ThreadPool(n_processor) as p:
                test_multi = partial(input_spectra, args = args, inv = inv, Start = str(Start), i = i)   
                
                if args.sta != 'P2' and args.sta != 'P3' and args.sta != 'DZA11' and args.sta != 'DZA12' and args.sta != 'TERZ':
                
                    p.map(test_multi, ['HHE', 'HHN', 'HHZ'])  
                    
                elif args.sta == 'P2' or args.sta == 'P3' or args.sta == 'DZA11' or args.sta == 'DZA12' or args.sta == 'TERZ':
                    
                    p.map(test_multi, ['HH1', 'HH2', 'HHZ'])  
      
            
        elif cha != 'HH*':
            
            input_spectra(cha, args, inv, Start, T, i)
            
        else:
            
            print('SKIP DAY')
  
        #if len(st) == 3:
        #
        #        ff, _, S_z = spectrogram(np.array(st[2].data), fs=fs, window='hann', nperseg=fs*T, noverlap=fs*T/2)
        #        ff, _, S_e = spectrogram(np.array(st[0].data), fs=fs, window='hann', nperseg=fs*T, noverlap=fs*T/2)
        #        ff, _, S_n = spectrogram(np.array(st[1].data), fs=fs, window='hann', nperseg=fs*T, noverlap=fs*T/2)
        #        
        #        
        #        if i == 0: 
        #            
        #            a, b        = np.shape(S_z)
        #            S_tot_z     = np.empty((a, 0), dtype = 'float64')
        #            S_tot_z     = np.append(S_tot_z, S_z, axis=1)
        #            
        #            a,b         = np.shape(S_e)
        #            S_tot_e     = np.empty((a, 0), dtype = 'float64')
        #            S_tot_e     = np.append(S_tot_e, S_e, axis=1)               
        #            
        #            a,b         = np.shape(S_n)
        #            S_tot_n     = np.empty((a, 0), dtype = 'float64')
        #            S_tot_n     = np.append(S_tot_n, S_n, axis=1)                
        #            
        #            
        #        elif i >= 1:
        #            
        #            try:
        #                S_tot_z
        #                S_tot_e
        #                S_tot_n
        #                
        #            except NameError:
        #                
        #                a,b        = np.shape(S_z)
        #                S_tot_z     = np.empty((a, 0), dtype = 'float64')  
        #                
        #                a,b         = np.shape(S_e)
        #                S_tot_e     = np.empty((a, 0), dtype = 'float64')  
        #                
        #                a,b         = np.shape(S_n)
        #                S_tot_n     = np.empty((a, 0), dtype = 'float64')
        #                
        #            S_tot_z     = np.append(S_tot_z, S_z, axis=1)
        #            S_tot_e     = np.append(S_tot_e, S_e, axis=1)
        #            S_tot_n     = np.append(S_tot_n, S_n, axis=1)
        #       
        #        
        #        if i == args.days - 1: plot = True
        #        
        #        plot_n_save(Start_0, args.days, bb, ff, _, T, S_tot_z, args.sta, loc, 'HHZ', write = False, plot = plot)
        #        plot_n_save(Start_0, args.days, bb, ff, _, T, S_tot_e, args.sta, loc, 'HHE', write = False, plot = plot)
        #        plot_n_save(Start_0, args.days, bb, ff, _, T, S_tot_n, args.sta, loc, 'HHN', write = False, plot = plot)
        #           

        
    
