def rose_plot(ax, angles, bins, density=False, offset=0, lab_unit="degrees",
              start_zero=True, **param_dict):
    """
    Plot polar histogram of angles on ax. ax must have been created using
    subplot_kw=dict(projection='polar'). Angles are expected in radians.
    """
    # Wrap angles to [-pi, pi)
    angles = (angles + np.pi) % (2*np.pi) - np.pi
    #print(angles*180/np.pi)

    # Set bins symetrically around zero
    if start_zero:
        # To have a bin edge at zero use an even number of bins
        if bins % 2:
            bins += 1
        bins = np.linspace(-np.pi, np.pi, num=bins+1)
        
    #print(bins)

    # Bin data and record counts
    count, bin = np.histogram(angles, bins=bins)
    #print(bin*180/np.pi)
    # Compute width of each bin
    widths = np.diff(bin)

    # By default plot density (frequency potentially misleading)
    if density is None or density is True:
        # Area to assign each bin
        area = count / angles.size
        # Calculate corresponding bin radius
        radius = ((area / np.pi)**.5)
    else:
        radius = count

    # Plot data on ax
    ax.bar(bin[:-1], radius, zorder=1, align='edge', width=widths,
           color='r', edgecolor = 'k', fill=True, linewidth=1)

    # Set the direction of the zero angle
    #ax.set_theta_offset(offset)

    # Remove ylabels, they are mostly obstructive and not informative
    ax.set_yticklabels([])

    if lab_unit == "radians":
        label = ['$0$', r'$\pi/4$', r'$\pi/2$', r'$3\pi/4$',
                  r'$\pi$', r'$5\pi/4$', r'$3\pi/2$', r'$7\pi/4$']
        ax.set_xticklabels(label)
  
def rotation_matrix(alpha):
    
    rot_mat = np.array(((np.cos(alpha), np.sin(alpha)), (-np.sin(alpha), np.cos(alpha))))
    return rot_mat

def rotation(traccia1, traccia2, alpha):
    
    rotated = []
    rot_mat = rotation_matrix(alpha)
    
    for i in range(len(traccia1)):

        bridge = np.array([traccia1[i], traccia2[i]])
        rotated.append(rot_mat.dot(bridge))
        
    return np.transpose(np.array(rotated))



import warnings
warnings.filterwarnings('ignore')
import obspy
import math
from obspy import read
import pylab
import numpy as np
pylab.rcParams['figure.figsize'] = (12.0, 8.0)
from obspy import UTCDateTime
from obspy.clients.filesystem.sds import Client
root = '/archive/AMIATA'
archive = Client(root)
from obspy import read_inventory
import glob
from datetime import datetime
from scipy.ndimage.filters import gaussian_filter
from scipy.fft import fft, fftfreq
#from obspy.clients.fdsn import Client as remoto
import matplotlib.pyplot as plt
from IPython.display import display, clear_output
#client = remoto("INGV")

stations = ['SOE0', 'SOE1', 'SOE2']
#stations  = ['AGLI']
#times    = ['2020-11-3T00:00', '2020-11-22T00:00', '2020-9-25T00:00', '2020-9-22T00:00', '2021-1-22T00:00', '2021-1-20T00:00', '2021-1-14T00:00']
times    = ['2020-1-1T00:00', '2020-4-1T00:00', '2020-7-1T00:00', '2020-10-1T00:00']
ore    = [24*120, 24*120, 24*120, 24*120]

#times    = ['2020-4-1T00:00']
#ore    = [24*30]


inv  = read_inventory('/homes/digiovanni/Sos_Enattos.xml')
inv += read_inventory('/scratch/SosEnattos/Rosmini/SOE1.seed')
#inv = client.get_stations(network="IV", station="AGLI", loc="", channel="HH*", level="response")


for k, t in enumerate(times):
    
    Start_0 = UTCDateTime(t)
    hours = ore[k]

    for sta in stations:
        
        phase_shift = np.zeros(hours)
        max_angle   = np.zeros(hours)
        f_dominant  = np.zeros(hours)
        inizio      = []

        for end in range(hours):
        
            Start = Start_0 + end*3600
            inizio.append(Start.datetime)
        
            try:
                               
                clear_output(wait=True)
                display(''+str(sta) + ' Ora: '+str(end)+' '+str(Start)+' '+ str(Start + 3600))
            
                st = archive.get_waveforms(station=sta,channel='HH?',network='ET',location='',starttime=Start,endtime=Start + 3600)
                st.remove_response(output="VEL", water_level=60,inventory=inv)
                st.detrend('demean')
                
                #st = client.get_waveforms(station=sta,channel='HH?',network='IV',location='',starttime=Start,endtime=Start + 3600)
                #st.remove_response(output="VEL", water_level=60,inventory=inv)
                #st.detrend('demean')
                
                tre             = st[0]
                trn             = st[1]
                trz             = st[2]
                delta           = float(tre.stats.delta)
                samp            = trz.stats.sampling_rate
                T               = 1/100
                N               = len(trz.data)
                
                if sta == 'SOE0':
                
                    FFT_E      = fft(tre.data)
                    FFT_N      = fft(trn.data)
                    
                elif sta == 'SOE1':
                    
                    new_tr = rotation(np.array(trn.data), np.array(tre.data), 195*np.pi/180)

                    FFT_E      = fft(new_tr[1])
                    FFT_N      = fft(new_tr[0])
                    
                elif sta == 'SOE2':
                    
                    new_tr = rotation(np.array(trn.data), np.array(tre.data), 139*np.pi/180)

                    FFT_E      = fft(new_tr[1])
                    FFT_N      = fft(new_tr[0])

                FFT_Z           = fft(trz.data)
                FFT_E           = FFT_E[0:N//2]
                FFT_N           = FFT_N[0:N//2]
                FFT_Z_real      = FFT_Z * np.conjugate(FFT_Z)
                FFT_Z_real      = (2.0*delta/N * np.abs(FFT_Z_real[0:N//2]))
                FFT_Z_real      = gaussian_filter(FFT_Z_real, sigma = 10)
                frequency       = fftfreq(N, T)[:N//2]
                delta           = float(trz.stats.delta)
                phase_Z         = np.angle(FFT_Z[0:N//2], deg = False)
                index_micro     = np.where((frequency < 1) & (frequency > 0.1))
                mask_seis_Z     = np.zeros(len(FFT_Z_real), dtype = bool)
                for i in index_micro[0]: mask_seis_Z[i] = True
                range_Z         = FFT_Z_real[mask_seis_Z]
                f_z             = frequency[mask_seis_Z]
                max_arg         = np.argmax(range_Z)
                f_dominant[end] = f_z[max_arg]
                
                if f_dominant[end] <= 0.11:
                    
                    stb = archive.get_waveforms(station=sta,channel='HHZ',network='ET',location='',starttime=Start,endtime=Start + 3600)
                    stb.remove_response(output="ACC", water_level=60,inventory=inv)
                    stb.detrend('demean')
                    
                    trzb              = stb[0]
                    FFT_Zb            = fft(trzb.data)
                    FFT_Zb_real       = FFT_Zb * np.conjugate(FFT_Zb)
                    FFT_Zb_real       = (2.0*delta/N * np.abs(FFT_Zb_real[0:N//2]))
                    FFT_Zb_real       = gaussian_filter(FFT_Zb_real, sigma = 10)                    
                    #plt.plot(frequency, (FFT_Zb_real), color = 'r')
                    index_micro       = np.where((frequency < 1) & (frequency > 0.1))
                    mask_seis_Z       = np.zeros(len(FFT_Zb_real), dtype = bool)
                    for i in index_micro[0]: mask_seis_Z[i] = True
                    range_Z           = FFT_Zb_real[mask_seis_Z]
                    f_z               = frequency[mask_seis_Z]
                    max_arg           = np.argmax(range_Z)
                    f_dominant[end]   = f_z[max_arg]                
                
                prova0     = np.where((frequency < f_dominant[end] + 0.01) & (frequency > f_dominant[end] - 0.01))
                
                mask_seis_E  = np.zeros(len(FFT_E), dtype = bool)
                mask_seis_N  = np.zeros(len(FFT_N), dtype = bool)
                for i in prova0[0]: mask_seis_E[i] = True
                for i in prova0[0]: mask_seis_N[i] = True
                range_E    = FFT_E[mask_seis_E]
                range_N    = FFT_N[mask_seis_N]
                #print(f_dominant)
                #max_angle = 0
                
                max_I   = 0                    
                angles  = np.linspace(0,360,360, dtype='int64')
                somma   = np.zeros(len(angles))
                
                for i,k in enumerate(angles):
                
                    radians = np.pi * k / 180
                
                    for j in range(len(range_E)):
                        
                        complex_sum = range_N[j] * np.cos(radians) + range_E[j] * np.sin(radians)     
                        somma[i]   += np.real(complex_sum)**2 + np.imag(complex_sum)**2
                        
                max_I          = np.amax(somma)
                I              = np.where(somma == np.amax(somma))
                max_angle[end] = angles[np.argmax(somma)] * np.pi/180
                        
                index            = np.where(frequency == f_dominant[end])
                to_ratio_1       = np.imag(FFT_N[index]) * np.cos(max_angle[end]) + np.imag(FFT_E[index]) * np.sin(max_angle[end])
                to_ratio_2       = np.real(FFT_N[index]) * np.cos(max_angle[end]) + np.real(FFT_E[index]) * np.sin(max_angle[end])
                phase_H          = np.arctan2(to_ratio_1,to_ratio_2)
                   
                phase_shift[end] = phase_H - phase_Z[index]
                #print('PHASE H: ' + str(phase_H) + ' phase_Z: ' + str(phase_Z[max_arg]))
                #print('PHASE SHIFT: ' + str(phase_shift[end]))
                #print('AZIMUTH: ' + str(max_angle[end]*180/np.pi))
                #print(phase_Z[np.argmax(FFT_Z_real)], phase_H*180/np.pi)
                #print(phase_H*180/np.pi - phase_Z[np.argmax(FFT_Z_real)])
                
                #if phase_shift[end] < -2*np.pi: phase_shift[end] += 2*np.pi
                #if phase_shift[end] > np.pi   : phase_shift[end]  = phase_shift[end] - 2*np.pi
                #if phase_shift[end] < -np.pi  : phase_shift[end]  = phase_shift[end] + 2*np.pi
                    
                if phase_shift[end] > np.pi:                    
                    phase_shift[end] -= 2*np.pi
                    
                elif phase_shift[end] < -np.pi:
                    phase_shift[end] += 2*np.pi
                
                if phase_shift[end] < 0 and max_angle[end] + np.pi < 2*np.pi: 
                    
                    max_angle[end]   += np.pi
                    phase_shift[end] += np.pi
                    
                elif phase_shift[end] < 0 and max_angle[end] + np.pi > 2*np.pi:
                    
                    max_angle[end]   -= np.pi
                    phase_shift[end] += np.pi
                    
                #print('PHASE SHIFT AFTER: ' + str(phase_shift[end]))
                #print('AZIMUTH AFTER: ' + str(max_angle[end]*180/np.pi))
                
            except:
            
                   print('SKIP')
                #    #max_angle[end]   = np.nan
                #    #phase_shift[end] = np.nan
                
        
                
        screen_angle     = np.where((phase_shift*180/np.pi < 120) & (phase_shift*180/np.pi > 60))
        mask_ang         = np.zeros(len(phase_shift), dtype = bool)
        for i in screen_angle[0]: mask_ang[i] = True
        range_ang        = max_angle[mask_ang]
        
        screen_phase     = np.where(phase_shift != 0)
        mask_ph          = np.zeros(len(phase_shift), dtype = bool)
        for i in screen_phase[0]: mask_ph[i] = True
        range_ph         = phase_shift[mask_ph]
        
        fig = plt.figure(figsize=(10,10))
        ax = plt.subplot(projection='polar')
        rose_plot(ax, range_ang, bins = 24)
        ax.set_theta_zero_location("N")
        ax.set_theta_direction(-1)
        plt.tight_layout()
        plt.savefig('./plots/'+str(sta)+'_'+str(t)+'_+'+str(hours)+'_hours.png')
        plt.clf()
        
        fig = plt.figure(figsize=(10,5))  
        pylab.rcParams.update({'font.size': 15})
        plt.plot(inizio, f_dominant, marker = 'o', linestyle = ' ', color = 'k')
        plt.ylim([0,1])
        plt.xlabel('Time', fontsize = 20, labelpad = 20)
        plt.ylabel('Frequency [Hz]', fontsize = 20)
        plt.xticks(rotation=90)
        plt.tight_layout()
        plt.grid()
        plt.savefig('./plots/'+str(sta)+'dominant_frequencies_'+str(t)+'_+'+str(hours)+'_hours.png')
        plt.clf()
        
        fig = plt.figure(figsize=(10,5))  
        pylab.rcParams.update({'font.size': 15})
        plt.hist(range_ph, bins = 12, color = 'b')
        plt.xticks(ticks=[0, np.pi/4, np.pi/2, 3/4*np.pi, np.pi], labels=['0', 'pi/4', 'pi/2', '3/4pi', 'pi'])
        plt.xlabel('H-Z phase shift', fontsize = 20)
        plt.ylabel('Counts', fontsize = 20)
        plt.grid()
        plt.tight_layout()
        plt.savefig('./plots/'+str(sta)+'phase_shift_distr_'+str(t)+'_+'+str(hours)+'_hours.png')
        plt.clf()