def rose_plot(ax, angles, bins, density=False, offset=0, lab_unit="degrees",
              start_zero=True, **param_dict):
    
    import numpy as np
    
    """
    Plot polar histogram of angles on ax. ax must have been created using
    subplot_kw=dict(projection='polar'). Angles are expected in radians.
    """
    # Wrap angles to [-pi, pi)
    angles = (angles + np.pi) % (2*np.pi) - np.pi
    #print(angles*180/np.pi)

    # Set bins symetrically around zero
    if start_zero:
        # To have a bin edge at zero use an even number of bins
        if bins % 2:
            bins += 1
        bins = np.linspace(-np.pi, np.pi, num=bins+1)
        
    #print(bins)

    # Bin data and record counts
    count, bin = np.histogram(angles, bins=bins)
    #print(bin*180/np.pi)
    # Compute width of each bin
    widths = np.diff(bin)

    # By default plot density (frequency potentially misleading)
    if density is None or density is True:
        # Area to assign each bin
        area = count / angles.size
        # Calculate corresponding bin radius
        radius = ((area / np.pi)**.5)
    else:
        radius = count

    # Plot data on ax
    ax.bar(bin[:-1], radius, zorder=1, align='edge', width=widths,
           color='r', edgecolor = 'k', fill=True, linewidth=1)

    # Set the direction of the zero angle
    #ax.set_theta_offset(offset)

    # Remove ylabels, they are mostly obstructive and not informative
    ax.set_yticklabels([])

    if lab_unit == "radians":
        label = ['$0$', r'$\pi/4$', r'$\pi/2$', r'$3\pi/4$',
                  r'$\pi$', r'$5\pi/4$', r'$3\pi/2$', r'$7\pi/4$']
        ax.set_xticklabels(label)
  
#######################

def rotation_matrix(alpha):
    
    import numpy as np
    
    rot_mat = np.array(((np.cos(alpha), np.sin(alpha)), (-np.sin(alpha), np.cos(alpha))))
    return rot_mat

def rotation(traccia1, traccia2, alpha):
    
    import numpy as np
    
    rotated = []
    rot_mat = rotation_matrix(alpha)
    
    for i in range(len(traccia1)):

        bridge = np.array([traccia1[i], traccia2[i]])
        rotated.append(rot_mat.dot(bridge))
        
    return np.transpose(np.array(rotated))

def noise_trend(path_to_psd, settimane):

    import numpy as np
     
    s0Z = read_ppsd(path_to_psd)    
        
    f_band                    = np.where((1/s0Z._period_binning[2] < 1) & (1/s0Z._period_binning[2] > 0.1))
    l                         = len(s0Z._period_binning[2])
    mask_f                    = np.zeros(l, dtype = bool)
    for i in f_band[0]: mask_f[i]    = True
    rango                     = 1/s0Z._period_binning[2][mask_f]
    tztime0                   = [t.datetime for t in s0Z.times_processed]
    media_settimanale         = np.zeros(len(settimane))
    perce_settimanale         = np.zeros(len(settimane))
    perce_settimanale_90      = np.zeros(len(settimane))
    giorni                    = []
    average_ts                = 0
    to_percentiles_02         = []
    
    for k in rango:
    
        w2f                  = 2.0 * np.pi * k
        s0z_psd7,g2,g3,g4    = s0Z.extract_psd_values(period=1/k)
        to_percentiles_02.append(10**(np.array(s0z_psd7)/20)/w2f)
        average_ts          += 10**(np.array(s0z_psd7)/20)/w2f

    average_ts    = np.array(average_ts)/len(rango)
    percentile_10 = np.percentile(to_percentiles_02, [10], axis = 0)
    percentile_90 = np.percentile(to_percentiles_02, [90], axis = 0)

    for s,w in enumerate(settimane): 
        
        itime      = [i for i,t in enumerate(tztime0) if t.isocalendar()[1] == w]
        
        new_time_0 = [tztime0[i] for i in itime]
        new_ts_0   = [average_ts[i] for i in itime]
        new_perc   = [percentile_10[0][i] for i in itime]
        
        media_settimanale[s] = np.mean(new_ts_0)
        perce_settimanale[s] = np.mean(new_perc)
        perce_settimanale_90[s] = np.mean(new_perc)
        new_perc   = [percentile_90[0][i] for i in itime]
        perce_settimanale_90[s] = np.mean(new_perc)
        giorni.append(new_time_0[0])
        
    return giorni, media_settimanale, perce_settimanale, perce_settimanale_90

def micro_peak(path_to_psd, settimane):
    
    import numpy as np
    
    s0Z = read_ppsd(path_to_psd) 
      
    a,b                       = np.shape(s0Z._binned_psds)
    tztime                    = [t.datetime for t in s0Z.times_processed]
    select                    = np.where((s0Z._period_binning[2] < 10) & (s0Z._period_binning[2] > 1))
    mask_f                    = np.zeros(b, dtype = bool)
    mask_seis                 = np.zeros(b, dtype = bool)
    for i in select[0]: mask_f[i]    = True
    for i in select[0]: mask_seis[i] = True
    media_settimanale         = np.zeros(len(settimane))
    percentile_10             = np.zeros(len(settimane))
    percentile_90             = np.zeros(len(settimane))
    giorni                    = []
    average_ts                = np.array(s0Z._binned_psds)
    micro_peak                = np.zeros(len(settimane))
    mean_FWHM                 = np.zeros(len(settimane))
    
    for s,w in enumerate(settimane): 
            
        itime      = [i for i,t in enumerate(tztime) if t.isocalendar()[1] == w]
        
        new_time = [tztime[i] for i in itime]
        new_time = np.array(new_time)
        new_ts   = [average_ts[i] for i in itime]
        new_ts   = np.array(new_ts)
        #print(np.shape(new_ts))
        #print(np.shape(new_time))
        
        a,b = np.shape(new_ts)
        
        to_percentiles_02         = []
        
        for i in range(a):
            
            maxi                = np.argmax(new_ts[i][mask_seis])
            bridge_times        = np.array(s0Z._period_binning[2])
            bridge_times        = bridge_times[mask_f]
            micro_peak[s]   += bridge_times[maxi]
            to_percentiles_02.append(bridge_times[maxi])
            FWHM                = np.where(10**(new_ts[i][mask_seis]/10) > np.max(10**(new_ts[i][mask_seis]/10)/2))
            mean_FWHM[s]    += 1/bridge_times[FWHM[0][len(np.transpose(FWHM)) - 1]] - 1/bridge_times[FWHM[0][0]]
            #print(bridge_times[maxi])
            #peak_value_21.append(new[maxi])
        
        micro_peak[s] /= a
        mean_FWHM[s]  /= a
        giorni.append(new_time[0])
        
        percentile_10[s] = np.percentile(to_percentiles_02, [10], axis = 0)[0]
        percentile_90[s] = np.percentile(to_percentiles_02, [90], axis = 0)[0]
        
    return giorni, 1/micro_peak, 1/percentile_10, 1/percentile_90

def make_rms_trend(path_to_psd_1, path_to_psd_2, settimane):
    
    import numpy as np
    from scipy.signal import find_peaks
    from scipy.signal import savgol_filter
 
    s0Z = read_ppsd(path_to_psd_1) 
    s2Z = read_ppsd(path_to_psd_2) 

    tztime                 = [t.datetime for t in s0Z.times_processed]
    tztime_2               = [t.datetime for t in s2Z.times_processed]
    average_ts_disp        = 0
    average_ts_disp_2      = 0
    f_band                 = np.where((1/s0Z._period_binning[2] <= 1) & (1/s0Z._period_binning[2] >= 1e-1))
    l                      = len(s0Z._period_binning[2])
    mask_f                 = np.zeros(l, dtype = bool)
    for i in f_band[0]: mask_f[i]    = True
    rango                  = 1/s0Z._period_binning[2][mask_f]
    media_settimanale      = np.zeros(len(settimane))
    giorni                 = []
    average_ts             = np.array(s0Z._binned_psds)
    rms_peak               = np.zeros(len(settimane))
    rms_mean               = np.zeros(len(settimane))
    percentile_10          = np.zeros(len(settimane))
    percentile_90          = np.zeros(len(settimane))
    mean_FWHM              = np.zeros(len(settimane))

    for k in rango:

        #w2f                 = (2.0 * np.pi * k)
        #print(k)
        s0z_psd7,g2,g3,g4    = s0Z.extract_psd_values(period=1/k)
        s2z_psd7,g2,g3,g4    = s2Z.extract_psd_values(period=1/k)  
        #plt.plot(s2z_psd7)
        #plt.show()
        #average_ts_disp_19 += np.sqrt(np.array(10**(np.array(s0z_psd7)/10))/w2f**4)   
        #average_ts_disp_19   += np.sqrt(np.array(10**(np.array(s0z_psd7)/10)))
        #average_ts_disp_19_2 += np.sqrt(np.array(10**(np.array(s2z_psd7)/10)))

        average_ts_disp   += (np.array(10**(np.array(s0z_psd7)/10)))
        average_ts_disp_2 += (np.array(10**(np.array(s2z_psd7)/10)))


    average_ts_disp   = np.sqrt(np.array(average_ts_disp))#/len(rango))
    average_ts_disp_2 = np.sqrt(np.array(average_ts_disp_2))#/len(rango))

    ip_0,_     = find_peaks(average_ts_disp, prominence = 2e-4)
    time_p_0   = [s0Z.times_processed[i] for i in ip_0]
    points_p_0 = [average_ts_disp[i] for i in ip_0]

    ip_2,_     = find_peaks(average_ts_disp_2, prominence = 2e-4)
    time_p_2   = [s2Z.times_processed[i] for i in ip_2]
    points_p_2 = [average_ts_disp_2[i] for i in ip_2]

    common_times = []

    for i,t in enumerate(time_p_0):

        for j, t_2 in enumerate(time_p_2):

            if t == t_2 - 3600/2 or t == t_2 + 3600/2 or t == t_2:

                common_times.append(i)

    mask0 = np.ones(len(average_ts_disp_2), dtype = bool)

    # STEP 1

    prova0 = np.where(average_ts_disp_2 > np.mean(average_ts_disp_2) + 3*np.std(average_ts_disp_2))
    for i in prova0[0]: mask0[i] = False   
    average_ts_mask =  average_ts_disp_2[mask0]

    # STEP 2    

    prova0 = np.where(average_ts_disp_2 > np.mean(average_ts_mask) + 3*np.std(average_ts_mask))   
    for i in prova0[0]: mask0[i] = False    
    average_ts_mask =  average_ts_disp_2[mask0] 

    # STEP 3

    prova0 = np.where(average_ts_disp_2 > np.mean(average_ts_mask) + 4*np.std(average_ts_mask))    
    for i in prova0[0]: mask0[i] = False
    average_ts_mask =  average_ts_disp_2[mask0] 

    average_ts_mask = savgol_filter(average_ts_mask, window_length = 21, polyorder = 2)

    tztime0_pow = np.array(tztime_2)
    tztime0     = tztime0_pow[mask0]

    for s,w in enumerate(settimane): 

        itime            = [i for i,t in enumerate(tztime0) if t.isocalendar()[1] == w]  
        new_time         = [tztime0[i] for i in itime]
        new_time         = np.array(new_time)
        new_ts           = [average_ts_mask[i] for i in itime]
        new_ts           = np.array(new_ts)
        rms_peak[s]      = np.amax(new_ts)
        rms_mean[s]      = np.mean(new_ts)
        percentile_10[s] = np.percentile(new_ts, [10], axis = 0)
        percentile_90[s] = np.percentile(new_ts, [90], axis = 0)
        giorni.append(new_time[0])
        
    return giorni, rms_mean, percentile_10, percentile_90

def read_nc(path_to_nc_file):
    
    import xarray as xr
    from netCDF4 import Dataset
    
    mare = Dataset(path_to_nc_file)
    ds = xr.open_dataset(xr.backends.NetCDF4DataStore(mare), chunks=-1)
    df = ds.to_dataframe()   
    time                   = mare.variables['time'][:]
    onda                   = mare.variables['VHM0'][:]
    longitude              = mare.variables['longitude'][:]
    latitude               = mare.variables['latitude'][:]
    
    return time, onda, longitude, latitude
    
    

def get_maxima(path_to_nc_file, settimane):
    
    import numpy as np
    from obspy import UTCDateTime
    
    tir_time, tir_onda, tir_longitude, tir_latitude = read_nc(path_to_nc_file) 
    tztime                     = []
    average_wave_h             = np.zeros(len(settimane))
    max_wave_h                 = np.zeros(len(settimane))
    min_wave_h                 = np.zeros(len(settimane))
    giorni                     = []
    percentile_10              = np.zeros(len(settimane))
    percentile_90              = np.zeros(len(settimane))
     
    for t in tir_time: tztime.append(UTCDateTime(t))
    
    for s,w in enumerate(settimane):
        
                    massimi    = []
                    medie      = []
                    itime      = [i for i,t in enumerate(tztime) if t.isocalendar()[1] == w]    
                    week_tir   = [tir_onda[i] for i in itime]    
                    week_tir   = np.transpose(week_tir)
                    new_time   = [tztime[i] for i in itime]
                    x,y,_        = np.shape(week_tir)
     
                    for i in range(x):
        
                        for j in range(y):
                
                            if np.amax(week_tir[i][j]) != -32767:
                        
                                massimi.append(np.amax(week_tir[i][j]))
                                medie.append(np.mean(week_tir[i][j]))
    
                    max_wave_h[s]     = np.max(massimi)
                    average_wave_h[s] = np.mean(medie) 
                    percentile_10[s]  = np.percentile(medie, [10], axis = 0)[0]
                    percentile_90[s]  = np.percentile(medie, [90], axis = 0)[0]
                    giorni.append(new_time[0].datetime)
                    
    return max_wave_h, average_wave_h, giorni, percentile_10, percentile_90
        
def single_points_mean(path_to_nc_file, settimane, coord_1, coord_2):
    
    import numpy as np
    from obspy import UTCDateTime
    
    tir_time, tir_onda, tir_longitude, tir_latitude = read_nc(path_to_nc_file)    
    #settimane                  = np.arange(20,27,1)
    tztime                     = [] 
    giorni                     = []
    average_wave_h             = np.zeros(len(settimane))
    
    for t in tir_time: tztime.append(UTCDateTime(t))
        
    for s,w in enumerate(settimane):
        
        medie      = []
        itime      = [i for i,t in enumerate(tztime) if t.isocalendar()[1] == w]    
        week_tir   = [tir_onda[i] for i in itime]    
        week_tir   = np.transpose(week_tir)
        new_time   = [tztime[i] for i in itime]
        
        average_wave_h[s] = np.mean(week_tir[coord_1][coord_2]) 
        giorni.append(new_time[0].datetime)
        
    return giorni, average_wave_h
    
    #pylab.rcParams['figure.figsize'] = (30, 10.0)
    #pylab.rcParams.update({'font.size': 25})
    #plt.plot(giorni,average_wave_h*1e-3, 'k', label = 'CMEMS data\nHigh correlation area', linewidth = 4)
    ##plt.xlim(datetime(2019,4,1),datetime(2021,5,12))    
    #plt.grid()
    #plt.show()
    
def read_ppsd(path_to_psd):
    
    import glob
    from obspy.signal import PPSD
    from obspy import read_inventory
    
    inv  = read_inventory('/homes/digiovanni/dataless/inv_nov_2021.xml')
    
    ppsdS0Z = glob.glob(path_to_psd)
    s0Z = PPSD.load_npz(ppsdS0Z[0],metadata=inv, allow_pickle=True)
    for f in ppsdS0Z:
        s0Z.add_npz(f, allow_pickle=True)
        
    return s0Z

def calculate_sea_corr(sea_waves, seismo_ts, mask_waves, mask_seismo):
    
    import numpy as np
    from obspy.signal.cross_correlation import xcorr_max,correlate
    
    onda                = np.transpose(sea_waves)
    x,y,z               = np.shape(onda)  
    correlation_map     = np.empty([y,x])
    average_wave_h      = np.empty([y,x])
    max_wave_h          = np.empty([y,x])
    min_wave_h          = np.empty([y,x])
    lunghezza_onde_alte = np.empty([y,x])  
    seismo_ts           = np.array(seismo_ts)    
    seismo_ts_mask      = seismo_ts[mask_seismo]
    
    for i in range(x):
    
        for j in range(y):
            
            if np.amax(onda[i][j]) != -32767:
                
                selected                     = onda[i][j]
                mask                         =  selected[mask_waves]
                c                            = correlate(mask, seismo_ts_mask, 0)
                shift, correlation_map[j][i] = xcorr_max(c)
                average_wave_h[j][i]         = np.mean(mask)
                max_wave_h[j][i]             = np.max(mask)
                min_wave_h[j][i]             = np.min(mask)
                lunghezza_onde_alte[j][i]    = len(mask[mask > 2e3])

            else:
            
                correlation_map[j][i]        = np.nan
                average_wave_h[j][i]         = np.nan
                max_wave_h[j][i]             = np.nan
                min_wave_h[j][i]             = np.nan
                lunghezza_onde_alte[j][i]    = np.nan
                
    return correlation_map, average_wave_h, max_wave_h, min_wave_h, lunghezza_onde_alte

def plot_corr_maps(lon, lat, llcrnrlon, llcrnrlat, urcrnrlon, urcrnrlat, correlation_map, average_wave_h, max_wave_h, min_wave_h, lunghezza_onde_alte, period, savefig = False):
    
    import pylab
    import numpy as np
    import matplotlib.pyplot as plt
    from obspy.imaging.cm import pqlx
    from mpl_toolkits.basemap import Basemap
    
    lon,lat  = np.meshgrid(lon,lat) 
    
    fig = plt.figure(figsize=(20,30))  
    pylab.rcParams.update({'font.size': 40})
    map = Basemap(llcrnrlon=llcrnrlon,llcrnrlat=llcrnrlat,urcrnrlon=urcrnrlon,urcrnrlat=urcrnrlat, epsg=3003, resolution = 'h')
    x, y     = map(lon, lat)
    map.drawcoastlines()
    map.fillcontinents(color='sandybrown',lake_color='white')
    map.pcolormesh(x, y, correlation_map, cmap=pqlx, vmin=0, vmax=1)
    map.drawparallels(np.arange(llcrnrlat,urcrnrlat,2),labels=[1,0,0,0])
    map.drawmeridians(np.arange(llcrnrlon,urcrnrlon,2.5),labels=[0,0,0,1])
    cbar = map.colorbar()
    cbar.set_label('Correlation', rotation=270, labelpad = 60, fontsize = 60)
    plt.title('Sea waves correlation (%1.2f Hz)' %(1/period), fontsize = 70, pad = 20)
    plt.xlabel('Longitude', fontsize = 60, labelpad = 70)
    plt.ylabel('Latititude', fontsize = 60, labelpad = 150)
    pylab.rcParams.update({'font.size': 50}) 
    plt.grid()    
    if savefig == True: pylab.savefig('/seawaves_plots/correlazione_mare_period_%f.png' %period, bbox_inches = "tight")
    plt.show()
    plt.clf()
    
    fig = plt.figure(figsize=(20,30))  
    pylab.rcParams.update({'font.size': 40})
    map = Basemap(llcrnrlon=llcrnrlon,llcrnrlat=llcrnrlat,urcrnrlon=urcrnrlon,urcrnrlat=urcrnrlat, epsg=3003, resolution = 'h')
    x, y     = map(lon, lat)
    map.drawcoastlines()
    map.fillcontinents(color='sandybrown',lake_color='white')
    map.pcolormesh(x, y, average_wave_h*1e-3, cmap=pqlx, vmin=0, vmax=2.5)
    map.drawparallels(np.arange(llcrnrlat,urcrnrlat,2),labels=[1,0,0,0])
    map.drawmeridians(np.arange(llcrnrlon,urcrnrlon,2.5),labels=[0,0,0,1])
    cbar = map.colorbar()
    cbar.set_label('Spectral significant\nwave heigth [m]', rotation=270, labelpad = 200, fontsize = 60)
    plt.title('Average Sea Wave heigth', fontsize = 70, pad = 20)
    plt.xlabel('Longitude', fontsize = 60, labelpad = 70)
    plt.ylabel('Latititude', fontsize = 60, labelpad = 150)
    plt.show()
    plt.clf()
    
    fig = plt.figure(figsize=(20,30))  
    pylab.rcParams.update({'font.size': 40})
    map = Basemap(llcrnrlon=llcrnrlon,llcrnrlat=llcrnrlat,urcrnrlon=urcrnrlon,urcrnrlat=urcrnrlat, epsg=3003, resolution = 'h')
    x, y     = map(lon, lat)
    map.drawcoastlines()
    map.fillcontinents(color='sandybrown',lake_color='white')
    map.pcolormesh(x, y, max_wave_h*1e-3, cmap=pqlx, vmin=0, vmax=5)
    map.drawparallels(np.arange(llcrnrlat,urcrnrlat,2),labels=[1,0,0,0])
    map.drawmeridians(np.arange(llcrnrlon,urcrnrlon,2.5),labels=[0,0,0,1])
    cbar = map.colorbar()
    cbar.set_label('Spectral significant\nwave heigth [m]', rotation=270, labelpad = 200, fontsize = 60)
    plt.title('Maximum Sea Wave heigth', fontsize = 70, pad = 20)
    plt.xlabel('Longitude', fontsize = 60, labelpad = 70)
    plt.ylabel('Latititude', fontsize = 60, labelpad = 150)
    plt.show()
    plt.clf()
    
    fig = plt.figure(figsize=(20,30))  
    pylab.rcParams.update({'font.size': 40})
    map = Basemap(llcrnrlon=llcrnrlon,llcrnrlat=llcrnrlat,urcrnrlon=urcrnrlon,urcrnrlat=urcrnrlat, epsg=3003, resolution = 'h')
    x, y     = map(lon, lat)
    map.drawcoastlines()
    map.fillcontinents(color='sandybrown',lake_color='white')
    map.pcolormesh(x, y, min_wave_h*1e-3, cmap=pqlx, vmin=0, vmax=5)
    map.drawparallels(np.arange(llcrnrlat,urcrnrlat,2),labels=[1,0,0,0])
    map.drawmeridians(np.arange(llcrnrlon,urcrnrlon,2.5),labels=[0,0,0,1])
    cbar = map.colorbar()
    cbar.set_label('Spectral significant\nwave heigth [m]', rotation=270, labelpad = 200, fontsize = 60)
    plt.title('Minimum Sea Wave heigth', fontsize = 70, pad = 20)
    plt.xlabel('Longitude', fontsize = 60, labelpad = 70)
    plt.ylabel('Latititude', fontsize = 60, labelpad = 150)
    plt.show()
    plt.clf()
    
    fig = plt.figure(figsize=(20,30))  
    pylab.rcParams.update({'font.size': 40})
    map = Basemap(llcrnrlon=llcrnrlon,llcrnrlat=llcrnrlat,urcrnrlon=urcrnrlon,urcrnrlat=urcrnrlat, epsg=3003, resolution = 'h')
    x, y     = map(lon, lat)
    map.drawcoastlines()
    map.fillcontinents(color='sandybrown',lake_color='white')
    map.pcolormesh(x, y, lunghezza_onde_alte/24, cmap = pqlx, vmin = 0, vmax = 20)
    map.drawparallels(np.arange(llcrnrlat,urcrnrlat,2),labels=[1,0,0,0])
    map.drawmeridians(np.arange(llcrnrlon,urcrnrlon,2.5),labels=[0,0,0,1])
    cbar = map.colorbar()
    cbar.set_label('Days', rotation=270, labelpad = 50, fontsize = 60)
    plt.title('Duration of high waves', fontsize = 70, pad = 20)
    plt.xlabel('Longitude', fontsize = 60, labelpad = 70)
    plt.ylabel('Latititude', fontsize = 60, labelpad = 150)
    plt.show()
    plt.clf()
    
def print_npz(codes_dict, inv, client):
    
    import calendar
    import numpy as np
    from obspy import read
    from obspy import UTCDateTime
    from obspy.clients.filesystem.sds import Client
    from obspy.signal import PPSD
    from obspy import read_inventory
    from obspy.clients.fdsn import Client as remoto
    
    if codes_dict['Start'].year == codes_dict['Stop'].year:    
    
        for s, sta in enumerate(codes_dict['Station']):
            
            for cha in codes_dict['Channels']:
        
                for day in range (codes_dict['Start'].julday,codes_dict['Stop'].julday + 1):
                    curday=UTCDateTime(year=codes_dict['Start'].year,julday=day)
                    print(str(sta) + ' ' + str(cha) + ' Processing ' ,curday)
                    
                    try:
                        
                        if codes_dict['Source'] == 'fdsn': inv = client.get_stations(network=codes_dict['Network'], station=sta, loc=codes_dict['Location'][s], channel=cha, level="response")
                            
                        st = client.get_waveforms(station=sta,channel=cha,network=codes_dict['Network'],location=codes_dict['Location'][s],
                                               starttime=curday,endtime=curday + 86400)
                            
                        tr = st.select(channel=cha)[0]
                        soe0 = PPSD(tr.stats, metadata=inv, skip_on_gaps=True)
                        soe0.add(st)
                        soe0.save_npz(filename=codes_dict['save_path']+'/'+soe0.id+'.'+str(curday.julday)+'.'+str(codes_dict['Start'].year)+'.npz')
                        
                    except:
                        
                        print('skip day')
                        
                    
    if codes_dict['Start'].year < codes_dict['Stop'].year:
        
        time_coord = {}
        time_coord['year_diff']   = codes_dict['Stop'].year - codes_dict['Start'].year
        time_coord['inizio']      = codes_dict['Start'].julday
        time_coord['anno_inizio'] = codes_dict['Start'].year
        
        if calendar.isleap(codes_dict['Start'].year) == True: time_coord['fine'] = 366
        else: time_coord['fine'] = 365
        
        for anni in range(time_coord['year_diff'] + 1):
            
            for s, sta in enumerate(codes_dict['Station']):
            
                for cha in codes_dict['Channels']:
            
                    for day in range (time_coord['inizio'] , time_coord['fine']  + 1):
                    
                        curday=UTCDateTime(year = time_coord['anno_inizio'],julday = day)
                        print(str(sta) + ' ' + str(cha) + ' Processing ' ,curday)
                        
                        try:
                            
                            if codes_dict['Source'] == 'fdsn': inv = client.get_stations(network=codes_dict['Network'], station=sta, loc=codes_dict['Location'][s], channel=cha, level="response")
                            
                            st = client.get_waveforms(station=sta,channel=cha,network=codes_dict['Network'],location=codes_dict['Location'][s],starttime=curday,endtime=curday + 86400)
                            
                            tr = st.select(channel=cha)[0]
                            soe0 = PPSD(tr.stats, metadata=inv, skip_on_gaps=True)
                            soe0.add(st)
                            soe0.save_npz(filename=codes_dict['save_path']+'/'+soe0.id+'.'+str(curday.julday)+'.'+str(time_coord['anno_inizio'])+'.npz')
                            
                        except:
                            
                            print('skip day')
                
            time_coord['inizio']       = 1
            time_coord['anno_inizio'] += 1
            
            if time_coord['anno_inizio'] == codes_dict['Stop'].year: time_coord['fine'] = codes_dict['Stop'].julday
            
            else:
                
                if calendar.isleap(time_coord['anno_inizio']) == True: time_coord['fine'] = 366
                else: time_coord['fine'] = 365
                
        
def deconData(inputSignal,dRatio,Fres,Fs,sensitivity,firstChecker):
    
    import numpy as np
    
    # input Signal
    # dRatio is the damping ratio, Fres is the geophone resonance frequency, Fs is the sampling frequency
    # sensitivity is in V/m/s and firstChecker is a flag for computing the transfer function
    N = len(inputSignal);
    FFTS = np.fft.fft(inputSignal)
    FFTSignal = FFTS[:int((N/2+1))]
    freq = np.arange(0,int(N/2+1),1)*250/N
    p=2
    if firstChecker == 1:
        omega = 2*np.pi*freq
        omegaO = 2*np.pi*Fres
        T = (-(omega*1j)**p)/((omegaO**2)+(omegaO*dRatio*2j)*omega-(omega**2))
        T_mag = abs(T)
        T_phase = np.angle(T)
    
        waterLevelInd = np.where(freq>0.1)[0]; waterLevelInd = waterLevelInd[0]
        waterLevel = T_mag[waterLevelInd]
        T_mag[T_mag < waterLevel] = waterLevel
        T = T_mag*np.exp(T_phase*1j)
        #np.save('Transfer.npy',T)
    else:
        T = np.load('Transfer.npy')
        
    T = T*sensitivity
    deconSignal = FFTSignal/T
    deconOut = make_FFT_symm(deconSignal)
    outSignal = np.fft.ifft(deconOut,N)
    realSignal = np.real(outSignal)
    return realSignal

def make_FFT_symm(Z):
    import numpy as np
    #make Z symmetric in the sense that the FFT of the output N will be real

    nVals = np.shape(Z)
    nVals = nVals[0]
    
    N = np.zeros(((nVals-1)*2),dtype=complex)

    N[0:nVals] = Z[0:nVals]
    N[nVals:(nVals-1)*2] = np.conj(np.flip( N[1:(nVals-1)] ) )
    return N 

def degreesToRadians(degrees):
    import numpy as np
    return degrees * np.pi / 180.;

def distanceInKmBetweenEarthCoordinates(lat1, lon1, lat2, lon2):
    
    import numpy as np
    
    earthRadiusKm = 6371;
  
    dLat = degreesToRadians(lat2-lat1)
    dLon = degreesToRadians(lon2-lon1)
  
    lat1r = degreesToRadians(lat1)
    lat2r = degreesToRadians(lat2)
  
    a = np.sin(dLat/2) * np.sin(dLat/2) + np.sin(dLon/2) * np.sin(dLon/2) * np.cos(lat1r) * np.cos(lat2r)
    c = 2 * np.arctan2(np.sqrt(a), np.sqrt(1-a))
    
    return earthRadiusKm * c


def parallel_corr(sta, sens_type, Start_0, channel, network, location, fs, days, per_flag, event_triggers, client):
    
    import numpy as np
    import sys
    import os
    from obspy import read
    from obspy import UTCDateTime
    from obspy.clients.filesystem.sds import Client
    from obspy import read_inventory
    from scipy.fft import fft, rfft, fftfreq
    from scipy.signal import csd
    sys.path.append('/homes/digiovanni/my_python_modules')
    from seismo_util import deconData, make_FFT_symm, degreesToRadians, distanceInKmBetweenEarthCoordinates, geophones_surveys_params
    
    T            = 1/fs
    seg_len      = 59
    step         = int(1440*days)   
    len_fact     = fs*step + 1
    frequency    = fftfreq(len_fact, T)[0:len_fact//2+1]
    fin_Xc       = 0
    count        = 0
    X_c_map      = []
    
    if sens_type == 'geop':
    
        _, _, _, deconParam_1hz, deconParam_5hz, sta_dict, ref = geophones_surveys_params(per_flag)


        print(sta)
    
        if sta_dict[sta] == 1:
        
            deconParam = deconParam_1hz
        
        elif sta_dict[sta] == 5:
        
            deconParam = deconParam_5hz
        
        fSamp = deconParam['fSamp']
        
    elif sens_type == 'broad':
        
        inv      = read_inventory('/archive/SARARRAY/dataless/7P_network.xml')
        
        if per_flag == 1: 
            
            ref       = 'P110'
            
        elif per_flag == 9: 
            
            ref       = 'P221'
            
        elif per_flag == 10: 
            
            ref       = 'P301'
    
    if per_flag == 9 and sens_type == 'broad': 
         
        filename     = '/homes/digiovanni/Correlations/P2/broadband_'+str(sta)+'_'+str(Start_0)+'_complex.npy'
        filename_map = '/homes/digiovanni/Correlations/P2/broadband_colormap_'+str(sta)+'_'+str(Start_0)+'.npy'  
        freq_file    = '/homes/digiovanni/Correlations/P2/broadband_frequencies.npy'
        
    elif per_flag == 9 and sens_type == 'geop': 
         
        filename     = '/homes/digiovanni/Correlations/P2/geophone_'+str(sta)+'_'+str(Start_0)+'_complex.npy'
        filename_map = '/homes/digiovanni/Correlations/P2/geophone_colormap_'+str(sta)+'_'+str(Start_0)+'.npy' 
        freq_file    = '/homes/digiovanni/Correlations/P2/geophone_frequencies.npy'
        
    elif per_flag == 7:
        
        filename     = '/homes/digiovanni/Correlations/P3/geophone_'+str(sta)+'_'+str(Start_0)+'_complex.npy'
        filename_map = '/homes/digiovanni/Correlations/P3/geophone_colormap_'+str(sta)+'_'+str(Start_0)+'.npy'
        freq_file    = '/homes/digiovanni/Correlations/P3/geophone_frequencies.npy'
        
    elif per_flag == 10:
        
        filename     = '/homes/digiovanni/Correlations/P3/broadband_'+str(sta)+'_'+str(Start_0)+'_complex.npy'
        filename_map = '/homes/digiovanni/Correlations/P3/broadband_colormap_'+str(sta)+'_'+str(Start_0)+'.npy'
        freq_file    = '/homes/digiovanni/Correlations/P3/broadband_frequencies.npy'
        
    elif per_flag == 1:
        
        filename     = '/homes/digiovanni/Correlations/P1/broadband_'+str(sta)+'_'+str(Start_0)+'_complex.npy'
        filename_map = '/homes/digiovanni/Correlations/P1/broadband_colormap_'+str(sta)+'_'+str(Start_0)+'.npy'
        freq_file    = '/homes/digiovanni/Correlations/P1/broadband_frequencies.npy'
    
    
    if os.path.isfile(filename) == False:
    
        for ore in range(step):
            
            flag = True
            
            Start   = Start_0 + ore*60
            
            if Start not in event_triggers:
                
                try:
                
                #print(Start)
                    st_ref  = client.get_waveforms(station=ref,channel=channel,network=network,location=location,starttime=Start,endtime=Start + seg_len)
                    st      = client.get_waveforms(station=sta,channel=channel,network=network,location=location,starttime=Start,endtime=Start + seg_len)
            
                    #print(len(st))
                    #print(st)
                    
                    if sens_type == 'geop':
                    
                        if len(st) > 1: traccia        = np.array(st[1].data)/1000
                        else:           traccia        = np.array(st[0].data)/1000
        
                        if len(st_ref) > 1: traccia_ref        = np.array(st_ref[1].data)/1000
                        else:               traccia_ref        = np.array(st_ref[0].data)/1000       
                     
                            
                        traccia        -= np.mean(traccia)
                        traccia_ref    -= np.mean(traccia_ref)
                        
                    
                        deconOut        = deconData(traccia,deconParam['dampRatio'],deconParam['fReso'],deconParam['fSamp'],deconParam['sen'],deconParam['tfCheck'])
                        deconOut_ref    = deconData(traccia_ref,deconParam['dampRatio'],deconParam['fReso'],deconParam['fSamp'],deconParam['sen'],deconParam['tfCheck'])
                    elif sens_type == 'broad':
                        
                        st.remove_response(output="VEL", water_level=600,inventory=inv)
                        st_ref.remove_response(output="VEL", water_level=600,inventory=inv)
                        
                        deconOut     = st[0].data
                        deconOut_ref = st_ref[0].data
                        fSamp        = int(st[0].stats.sampling_rate)
                    
                    f, Pxy = csd(deconOut, deconOut_ref, fs=fSamp, window='hanning', nperseg=32*fSamp, noverlap = 32*fSamp//2, detrend='constant', return_onesided=True, scaling='density')
                        
                    f, Pxx = csd(deconOut, deconOut, fs=fSamp, window='hanning', nperseg=32*fSamp, noverlap = 32*fSamp//2, detrend='constant', return_onesided=True, scaling='density')
                        
                    f, Pyy = csd(deconOut_ref, deconOut_ref, fs=fSamp, window='hanning', nperseg=32*fSamp, noverlap = 32*fSamp//2, detrend='constant', return_onesided=True, scaling='density')
                
                    Xc   = Pxy/np.sqrt(Pxx * Pyy)
                    check_f  = np.where((f > 0.1) & (f < 1))
                
                    if np.amax(np.abs(Xc[check_f]) > 0.7):  
                    
                        fin_Xc += Xc
                        count  += 1
                        X_c_map.append(Xc_real)
                    
                    
                    else: 
                    
                        fin_Xc += 0
                        #print('Skip for bad correlation')
            
                except: print('No data here')
            
            else:
                
                print('Skip for earthquake')
    
        
        if count != 0:
            
            np.save(filename, fin_Xc/count, allow_pickle=True, fix_imports=True)
            np.save(filename_map, X_c_map, allow_pickle=True, fix_imports=True)
            np.save(freq_file, f, allow_pickle=True, fix_imports=True)
            
        else:
            
            np.save(filename, 0, allow_pickle=True, fix_imports=True)
            np.save(filename_map, 0, allow_pickle=True, fix_imports=True)
        
        
                                           
        print(filename)

    else: print('File exhist, skipping')
    
def geophones_surveys_params(per_flag):
    
    import sys
    sys.path.append('/home/mdigiovanni/my_python_modules')
    from seismo_util import distanceInKmBetweenEarthCoordinates
    from obspy.clients.filesystem.sds import Client
    import numpy as np
    
    deconParam_1hz = {
    'tfCheck'   : 1,
    'fSamp'     : 250, 
    'Vref'     : 2.5,
    'gainFact'  : 16,
    'adcCount'  : 2**23,
    'sen'       : 276.8*40000/(40000+5500),
    'dampRatio' : 0.414, 
    'fReso'     : 1, 
    'winLen'    : 30*250,
    'winOv'     : 30*250/2,
    'fSt'       : 10,
    'fEnd'      : 20,  }


    deconParam_5hz = {
        'tfCheck'   : 1,
        'fSamp'     : 250,
        'Vref'     : 22.5,
        'gainFact'  : 16,
        'adcCount'  : 2**23,
        'sen'       : 80*40000/(40000+1850),
        'dampRatio' : 0.7,
        'fReso'     : 5,
        'winLen'    : 30*250,
        'winOv'     : 30*250/2,
        'fSt'       : 10,
        'fEnd'      : 20 }
    
    if per_flag == 7:
        
        root = r'/homes/digiovanni/GSSI_geophone_arrays/july_2021/GSSI_geop_jul_SDS/'
        client = Client(root)
        
        array_center = [9.465079, 40.578548]
        
        ref = '39708'
        
        sta_dict = {'39730': 1,
                '39706': 1,
                '39714': 1,
                '39717': 1,
                '39786': 1,
                '39760': 1,
                '39712': 1,
                '39758': 1,
                '39746': 1,
                '39767': 1,
                '39772': 1,
                '39707': 1,
                '39725': 1,
                '39702': 1,
                '39759': 1,
                '39726': 1,
                '39708': 5,
                '39728': 5,
                '39724': 5,
                '39765': 5,
                '39737': 5,
                '39757': 5,
                '39761': 5,
                '39756': 5,
                '39703': 5,
                '39749': 5,
                '39738': 5,
                '39754': 5,
                '39771': 5,
                '39710': 5,
                '39733': 5,
                '39711': 5,
                '39735': 5,
                '39784': 5,
                '39700': 5,
                '39778': 5,
                '39722': 5,
                '39729': 5,
                '39750': 5}
    
        x_coord = [9.46169,
                    9.46284,
                    9.46671,
                    9.46492,
                    9.46395,
                    9.46357,
                    9.46314,
                    9.46354,
                    9.46421,
                    9.46249,
                    9.46835,
                    9.4662,
                    9.46611,
                    9.46684,
                    9.46761,
                    9.4610239,
                    9.465079,
                    9.465222561,
                    9.464935439,
                    9.465079,
                    9.465394314,
                    9.465273874,
                    9.464884126,
                    9.464763686,
                    9.465079,
                    9.465597419,
                    9.465725456,
                    9.465366699,
                    9.464791301,
                    9.464432544,
                    9.464560581,
                    9.465079,
                    9.4657459,
                    9.4647857,
                    9.4638208,
                    9.4640498,
                    9.4644045,
                    9.4654536,
                    9.4661134]
    
    
        y_coord = [40.57812,
                    40.57803,
                    40.57801,
                    40.57973,
                    40.57955,
                    40.57923,
                    40.57868,
                    40.57809,
                    40.57802,
                    40.57873,
                    40.57856,
                    40.57865,
                    40.57823,
                    40.57952,
                    40.57829,
                    40.578383,
                    40.578548,
                    40.57848505,
                    40.57848505,
                    40.57867391,
                    40.57862581,
                    40.57834428,
                    40.57834428,
                    40.57862581,
                    40.57879981,
                    40.578862,
                    40.57843593,
                    40.57809425,
                    40.57809425,
                    40.57843593,
                    40.578862,
                    40.57905162,
                    40.5777339,
                    40.5776975,
                    40.5783471,
                    40.5789909,
                    40.5792908,
                    40.5797724,
                    40.5793303]
    
    
            
        #Start_0  = UTCDateTime('2021-7-10T00:00')
        
        x = np.empty(len(x_coord))
        y = np.empty(len(y_coord))
             
    #    for a, (i, j) in enumerate(zip(x_coord, y_coord)):
        
    #        distances    = distanceInKmBetweenEarthCoordinates(j, i, array_center[1], array_center[0])
    #        x[a]       = distances * np.cos(np.arctan2(j - array_center[1], i - array_center[0]))*1000
    #        y[a]       = distances * np.sin(np.arctan2(j - array_center[1], i - array_center[0]))*1000
            
    if per_flag == 9:
        
        root = r'/homes/digiovanni/GSSI_geophone_arrays/september_2021/GSSI_geop_sep_SDS/'
        client = Client(root)
         
        array_center = [9.34801, 40.52438]
        
        ref       = '39708'
        
        sta_dict = {'39765': 5,
                    '39758': 5,
                    '39707': 5,
                    '39735': 5,
                    '39729': 5,
                    '39761': 5,
                    '39700': 5,
                    '39711': 5,
                    '39725': 5,
                    '39760': 5,
                    '39702': 5,
                    '39706': 5,
                    '39730': 5,
                    '39771': 5,
                    '39728': 5,
                    '39756': 5,
                    #'39746': 5,
                    '39737': 5,
                    '39712': 5,
                    '39750': 5,
                    '39710': 5,
                    '39757': 5,
                    '39723': 5,
                    '39722': 5,
                    '39778': 5,
                    '39717': 5,
                    '39767': 5,
                    '39703': 5,
                    '39749': 5,
                    '39726': 5,
                    '39733': 5,
                    '39754': 5,
                    '39724': 5,
                    '39772': 5,
                    '39759': 5,
                    '39714': 5,
                    '39786': 5,
                    '39784': 5,
                    '39708': 5,
                    '39738': 5}
        
        
        x_coord = [9.348008,     
                   9.34798,
                   9.348042,
                   9.348071477,
                   9.347948523,
                   9.34801,
                   9.348145025,
                   9.34809345,
                   9.34792655,
                   9.347874975,
                   9.34801,
                   9.348232,
                   9.348286829,
                   9.3481332,
                   9.3478868,
                   9.347733171,
                   9.347788,
                   9.34801,
                   9.34801,
                   9.34904,
                   9.34968,
                   9.34965,
                   9.34894,
                   9.34797,
                   9.34694,
                   9.34634,
                   9.34631,
                   9.34695,
                   9.34828,
                   9.34863,
                   9.34891,
                   9.3487,
                   9.34826,
                   9.34773,
                   9.34728,
                   9.34711,
                   9.34727,
                   9.34772,
                   9.34801,
                   9.34845]
            
        y_coord = [40.524354,
                40.524396,
                40.524396,
                40.52435302,
                40.52435302,
                40.52443396,
                40.52441335,
                40.52429269,
                40.52429269,
                40.52441335,
                40.52448792,
                40.52451457,
                40.52433197,
                40.52418554,
                40.52418554,
                40.52433197,
                40.52451457,
                40.52459584,
                40.52571,
                40.52545,
                40.52477,
                40.52396,
                40.52312,
                40.52298,
                40.52328,
                40.52393,
                40.52482,
                40.52546,
                40.52499,
                40.52477,
                40.52436,
                40.52394,
                40.52369,
                40.52371,
                40.52396,
                40.52436,
                40.52473,
                40.52501,
                40.52438,
                40.52303]
        
    #styles   = ['solid', 'dashed', 'dashdot', 'dotted', (0,(1,10)), (0, (3, 10, 1, 10, 1, 10))]
    
        x = np.empty(len(x_coord))
        y = np.empty(len(y_coord))
    
    for i in range(len(x_coord)):
    
        distances  = distanceInKmBetweenEarthCoordinates(y_coord[i], x_coord[i], array_center[1], array_center[0])
        x[i]       = distances * np.cos(np.arctan2(y_coord[i] - array_center[1], x_coord[i] - array_center[0]))*1000
        y[i]       = distances * np.sin(np.arctan2(y_coord[i] - array_center[1], x_coord[i] - array_center[0]))*1000
        
        
    return x, y, client, deconParam_1hz, deconParam_5hz, sta_dict, ref
    



    
##########################################

def BLRMS(path_to_psd_1, path_to_psd_2, settimane):
    
    import numpy as np
    from scipy.signal import find_peaks
    from scipy.signal import savgol_filter

 
    s0Z = read_ppsd(path_to_psd_1) 
    s2Z = read_ppsd(path_to_psd_2) 


def plot_n_save(Start, days, bb, ff, tt, T, S_tot, sta, loc, channel, write, plot):
    
    import numpy as np
    import csv
    import matplotlib.pyplot as plt
    import warnings 
    warnings.filterwarnings('ignore')
    
    with open('/homes/digiovanni/LNM.txt') as inf:
        reader = csv.reader(inf, delimiter=" ")
        freq_NLM = list(zip(*reader))[0]
    
    with open('/homes/digiovanni/LNM.txt') as inf:
        reader = csv.reader(inf, delimiter=" ")
        NLM = list(zip(*reader))[1]

    freq_NLM = np.asarray(freq_NLM, dtype=np.float32)
    NLM      = np.asarray(NLM, dtype=np.float32)

    with open('/homes/digiovanni/HNM.txt') as inf:
        reader = csv.reader(inf, delimiter=" ")
        freq_HNM = list(zip(*reader))[0]
    
    with open('/homes/digiovanni/HNM.txt') as inf:
        reader = csv.reader(inf, delimiter=" ")
        HNM = list(zip(*reader))[1]

    freq_HNM = np.asarray(freq_HNM, dtype=np.float32)
    HNM      = np.asarray(HNM, dtype=np.float32)
    
    Shist= np.empty((len(bb)-1, len(ff)))
    for i in range(len(ff)):
        Shist[:,i] = np.histogram(np.sqrt(S_tot[i,:]), bins=bb)[0]/len(tt)
        
    bb = np.delete(bb, -1)
     
    Shist[Shist == 0] = np.nan
    
    if write == True: 
        
        save_path = '/homes/digiovanni/npy_spectra/global_'+str(sta)+'.'+str(loc)+'.'+str(channel)+'_Shist.npy'
        np.save(save_path, S_tot)
        print('Data saved to file: ', save_path)
                
    a = len(ff)
        
    percentile_10 = np.zeros(a)
    percentile_50 = np.zeros(a)
    percentile_90 = np.zeros(a)
        
    for i in range(a):
            
            somma  = np.nancumsum(Shist[:, i])
            vp_10  = np.amax(somma)*0.1
            vp_90  = np.amax(somma)*0.9
            vp_50  = np.amax(somma)*0.5
            
            p_10   = np.where(somma <= vp_10)
            p_90   = np.where(somma <= vp_90)
            p_50   = np.where(somma <= vp_50)
            
            #print(vp_10, len(p_10[0]), len(p_50[0]), len(p_90[0]))
            
            if len(p_10[0]) != 0:
                
                percentile_10[i] = bb[p_10[0][len(p_10[0]) - 1]]
                
            else:
                
                percentile_10[i] = 0
                
            if len(p_50[0]) != 0:
                
                percentile_50[i] = bb[p_50[0][len(p_50[0]) - 1]]
                
            else:
                
                percentile_50[i] = 0
            
            if len(p_90[0]) != 0:
                
                percentile_90[i] = bb[p_90[0][len(p_90[0]) - 1]]
                
            else:
                
                percentile_90[i] = 0
             
    
    if plot == True:
    
        plt.figure(figsize=(9,6))
        cmap = plt.get_cmap('RdYlBu_r')
        plt.pcolormesh(ff, bb, Shist, cmap=cmap)
        #plt.colorbar(label = 'Probability')
        plt.colorbar().set_label(label='Probability',size=30)
        plt.title('Spectra - '+str(sta)+'.'+str(loc)+'.'+str(channel), fontsize=30, pad = 20)
        plt.xlabel('Frequency [Hz]', fontsize=30)
        plt.ylabel(r"Amplitude [$\frac{(m/s)}{\sqrt{\rm Hz}}$]", fontsize=30)
        plt.xscale('log')
        plt.yscale('log')
        plt.plot(ff, (percentile_50), label = 'Median', linewidth = 3, color = 'orange')
        #plt.plot(f_240,10**(resp_z_gain_1x_amp_240/20)/(2*np.pi*f_240), label = 'self noise T120H + Centaur', color = 'green', linewidth = 6)
        plt.plot(freq_HNM,HNM, label = 'HNM', color = 'k', linestyle = '--', linewidth = 3)
        plt.plot(freq_NLM, NLM, label = 'NLM', color = 'k', linestyle = '--', linewidth = 3)
        plt.xlim([1/T, 20])
        plt.ylim([1e-11, 1e-5])
        plt.clim(0, np.nanmax(Shist)/np.sqrt(np.e))
        plt.grid()
        plt.legend()
        plt.tick_params(labelsize=20)
        plt.tight_layout()
        plt.savefig('plots/Histogram_'+str(sta)+'.'+str(loc)+'.'+str(channel)+'_'+str(Start)+'_'+str(days)+'.png')
        plt.show()
        plt.close()
        
            
        plt.figure(figsize=(9,6))
        plt.plot(ff, (percentile_10), color = 'r', label = '10th Percentile')
        plt.plot(ff, (percentile_50), color = 'k', label = '50th Percentile')
        plt.plot(ff, (percentile_90), color = 'b', label = '90th Percentile')
        plt.xlabel('Frequency [Hz]', fontsize=20)
        plt.ylabel(r"Seismic [(m/s)/$\sqrt{\rm Hz}$]", fontsize=20)
        plt.ylim([1e-11, 1e-5])
        plt.xlim([1/T, 20])
        plt.legend()
        plt.grid()
        plt.xscale('log')
        plt.yscale('log')
        plt.plot(freq_HNM,HNM, label = 'HNM', color = 'k', linestyle = '--', linewidth = 3)
        plt.plot(freq_NLM, NLM, label = 'NLM', color = 'k', linestyle = '--', linewidth = 3)
        plt.savefig('plots/Percentiles_'+str(sta)+'.'+str(loc)+'.'+str(channel)+'_'+str(Start)+'_'+str(days)+'.png')
        plt.close()
        
        print('Generated plots\n')
        print('plots/Histogram_'+str(sta)+'.'+str(loc)+'.'+str(channel)+'_'+str(Start)+'_'+str(days)+'.png')
    
    #plt.figure(figsize=(9,6))
    #plt.plot(ff, (percentile_10), color = 'r', label = '10th Percentile')
    #plt.plot(ff, (percentile_50), color = 'k', label = '50th Percentile')
    #plt.plot(ff, (percentile_90), color = 'b', label = '90th Percentile')
    #plt.xlabel('Frequency [Hz]', fontsize=20)
    #plt.ylabel(r"Seismic [(m/s)/$\sqrt{\rm Hz}$]", fontsize=20)
    #plt.ylim([1e-11, 1e-5])
    #plt.xlim([1/T, 20])
    #plt.legend()
    #plt.grid()
    #plt.xscale('log')
    #plt.yscale('log')
    #plt.plot(freq_HNM,HNM, label = 'HNM', color = 'k', linestyle = '--', linewidth = 3)
    #plt.plot(freq_NLM, NLM, label = 'NLM', color = 'k', linestyle = '--', linewidth = 3)
    #plt.savefig('plots/Percentiles_'+str(sta)+'.'+str(loc)+'.'+str(channel)+'_'+str(Start)+'_'+str(days)+'.png')
    #plt.close()
    
    print('Generated plots\n')
    print('plots/Percentiles_'+str(sta)+'.'+str(loc)+'.'+str(channel)+'_'+str(Start)+'_'+str(days)+'.png')
    
    file = open('plots/Percentiles_'+str(sta)+'.'+str(loc)+'.'+str(channel)+'_'+str(Start)+'_'+str(days)+'.txt','w') 
    #file.write('#frequency -  10th - 50th - 90th\n') 
    [file.write("%f %E %E %E\n" %(ff[k], percentile_10[k], percentile_50[k], percentile_90[k])) for k in range(len(ff))]
    file.close()
    
    print('Saved percentiles to: plots/Percentiles_'+str(sta)+'.'+str(loc)+'.'+str(channel)+'_'+str(Start)+'_'+str(days)+'.txt')
    
def spectra_scipy(trace, fs, T, sta, loc, channel):
    
    import numpy as np
    from scipy.signal import spectrogram
    
    S_tot = np.load('/homes/digiovanni/npy_spectra/global_'+str(sta)+'.'+str(loc)+'.'+str(channel)+'_Shist.npy')
    
    ff, tt, S   = spectrogram(np.array(trace), fs=fs, window='hann', nperseg=fs*T, noverlap=fs*T/2)
    a, b        = np.shape(S)
    print(a,b)
    
    
    S_tot       = np.append(S_tot, S, axis=1)
    
    return ff, tt, S_tot, S    

def standard_curves(code):
    
    import numpy as np
    
    #trillium 240
    
    f_noise = np.array([     
    0.0100, 
    0.0800, 
    0.1000, 
    0.2000, 
    1.0000, 
    2.0000, 
    3.5000, 
    5.0000, 
    8.0000, 
   10.0000, 
   20.0000])

    y_noise = np.array([
    -185.0000,
    -190.5000,
    -191.0000,
    -190.0000,
    -185.0000,
    -183.0000,
    -180.0000,
    -178.0000,
    -172.0000,
    -168.0000,
    -155.0000])
    
    #horizon 120
    
    f_h120 = np.array([     
    0.0100, 
    0.08254, 
    0.1000, 
    0.21544, 
    1.0000, 
    2.1544, 
    3.8312, 
    5.6234, 
    8.254, 
   10.0000, 
   21.544])
    
    y_h120 = np.array([  
    -184.56
    -188.85,
    -188.99,
    -189.16,
    -187.19,
    -184.2,
    -180.65,
    -177.51,
    -173.58,
    -171.28,
    -160.06])
    
    if code == 240:
    
        #Trillium 240
        zeros = np.array([0, 0, -90., -164.2, -3203.])
        poles = np.array([-0.01813 + 0.01803j, -0.01813 - 0.01803j, -124.9 + 0j,
             -197.5 + 256.1j, -197.5 - 256.1j, -569 + 1150j, -569 + -1150j])
        k = 4.532e5
        f_0 = 1
        S = 1196.6
    
    #Trillium Horizon 120
    #zeros = np.array([0, 0, -392., -1960., -1490 +1740j, -1490 -1740j])
    #poles = np.array([-0.03691+0.03702j, -0.03691-0.03702j, -343, -370+467j, 
    #                  -370-467j, -836+1522j, -836-1522j, -4900+4700j,
    #                 -4900-4700j, -6900, -15000])
    #k = 4.34493e17
    #f_0 = 1
    #S = 754.3
    
    if code == 120:
    
        #Trillium Horizon 120
        zeros = np.array([0, 0, -3.163000E+01, -1.600000E+02, -3.500000E+02, -3.177000E+03])
        poles = np.array([-3.661400E-02+3.705900E-02j, -3.661400E-02 -3.705900E-02j, -3.255000E+01
                     , -1.420000E+02, -3.640000E+02 + 4.040000E+02j, -3.640000E+02 -4.040000E+02j,
                     -1.260000E+03, -4.900000E+03 + 5.200000E+03j, -4.900000E+03 -5.200000E+03j,
                     -7.100000E+03 + 1.700000E+03j, -7.100000E+03 -1.700000E+03j])
        k = 8.318711e17
        f_0 = 1
        S = 1.202500e03
    
    f = np.array([0.0100 , 0.0125 , 0.0160 , 0.0200 , 0.0250 , 0.0315 ,
    0.040, 0.050 , 0.063 , 0.080 , 0.100 , 0.125 , 0.160 , 0.200 , 0.250 , 0.315 ,
    0.40 , 0.50 , 0.63 , 0.80 , 1.00 , 1.25 , 1.60 , 2.00 , 2.50 , 3.15 ,
    4.0 , 5.0 , 6.3 , 8.0 ,
    10.0 , 12.5 , 16.0 , 20.0 , 25.0 , 31.5 , 40.0])
    
    reale = np.empty(len(f))

    for e,u in enumerate(f):

        s = 2*np.pi*u*1j
        reale[e] = np.real(S * k * np.prod(s - zeros)/np.prod(s - poles))
    
    resp_z_gain_1x = np.array([-114.89 , -115.92 , -117.00 , -118.13 , -118.21 , -119.04 , -120.22 , -120.88 , -121.09 , -122.30 , -123.01 , -123.55 , -124.58 , -125.30 , -125.85 , -126.67 , -127.83 , -128.34 , -128.83 , -129.62 , -130.01 , -130.51 , -130.96 , -130.98 , -131.26 , -131.22 , -131.58 , -131.69 , -131.69 , -131.84 , -131.72 , -131.82 , -131.92 , -131.90 , -131.89 , -131.87 , -131.94])
    resp_z_gain_2x = np.array([-122.20 , -123.58 , -124.71 , -125.41 , -125.72 , -126.47 , -127.66 , -128.28 , -129.14 , -129.88 , -130.67 , -131.21 , -131.67 , -132.38 , -133.19 , -133.94 , -134.47 , -135.12 , -135.68 , -136.13 , -136.49 , -136.81 , -136.99 , -137.22 , -137.37 , -137.43 , -137.55 , -137.66 , -137.75 , -137.75 , -137.79 , -137.82 , -137.87 , -137.86 , -137.90 , -137.96 , -137.89])
    resp_z_gain_4x = np.array([-130.16 , -131.42 , -132.67 , -134.42 , -134.57 , -135.30 , -136.15 , -136.97 , -137.89 , -138.63 , -139.45 , -140.02 , -140.79 , -141.30 , -141.58 , -142.10 , -142.47 , -142.81 , -143.15 , -143.33 , -143.49 , -143.60 , -143.68 , -143.80 , -143.87 , -143.93 , -144.01 , -143.99 , -144.04 , -143.99 , -144.12 , -144.13 , -144.12 , -144.11 , -144.16 , -144.10 , -144.07])
    resp_z_gain_10x = np.array([-131.64 , -132.90 , -134.08 , -135.50 , -136.07 , -136.98 , -138.01 , -139.09 , -140.46 , -141.09 , -142.30 , -143.35 , -144.14 , -145.08 , -145.80 , -146.41 , -147.10 , -147.62 , -148.14 , -148.53 , -148.88 , -149.22 , -149.53 , -149.75 , -149.89 , -150.10 , -150.20 , -150.30 , -150.44 , -150.52 , -150.65 , -150.67 , -150.72 , -150.75 , -150.85 , -150.83 , -150.86])
    
    return f, f_noise, f_h120, y_h120, reale, y_noise, resp_z_gain_1x, resp_z_gain_2x, resp_z_gain_4x, resp_z_gain_10x
  


